@@ -18,7 +18,7 @@ from .agent_config import AgentConfig
 
 provider_to_default_model_name = {
     ModelProvider.OPENAI: "gpt-4.1-mini",
-    ModelProvider.ANTHROPIC: "claude-sonnet-4-0",
+    ModelProvider.ANTHROPIC: "claude-sonnet-4-5",
     ModelProvider.TOGETHER: "deepseek-ai/DeepSeek-V3",
     ModelProvider.GROQ: "openai/gpt-oss-20b",
     ModelProvider.BEDROCK: "us.anthropic.claude-sonnet-4-20250514-v1:0",
@@ -34,6 +34,8 @@ models_to_max_tokens = {
     "gpt-4.1-mini": 32768,
     "claude-sonnet-4-20250514": 64000,
     "claude-sonnet-4-0": 64000,
+    "claude-sonnet-4-5": 64000,
+    "claude-haiku-4-5": 32000,
     "deepseek-ai/deepseek-v3": 8192,
     "models/gemini-2.5-flash": 65536,
     "models/gemini-2.5-flash-lite": 65536,
@@ -83,6 +85,7 @@ def _create_llm_cache_key(role: LLMRole, config: Optional[AgentConfig] = None) -
         "tool_llm_model_name": config.tool_llm_model_name,
         "private_llm_api_base": config.private_llm_api_base,
         "private_llm_api_key": config.private_llm_api_key,
+        "private_llm_max_tokens": config.private_llm_max_tokens,
     }
 
     # Create a stable hash from the cache data
@@ -117,6 +120,57 @@ def _get_llm_params_for_role(
     return model_provider, model_name
 
 
+def _cleanup_gemini_clients() -> None:
+    """Helper function to cleanup Gemini client sessions."""
+    for llm in _llm_cache.values():
+        try:
+            # Check if this is a GoogleGenAI instance with internal client structure
+            if not hasattr(llm, '_client'):
+                continue
+
+            client = getattr(llm, '_client', None)
+            if not client:
+                continue
+
+            api_client = getattr(client, '_api_client', None)
+            if not api_client:
+                continue
+
+            async_session = getattr(api_client, '_async_session', None)
+            if not async_session:
+                continue
+
+            # Close the aiohttp session if it exists
+            try:
+                import asyncio
+                loop = asyncio.get_event_loop()
+                if not loop.is_closed():
+                    loop.run_until_complete(async_session.close())
+            except Exception:
+                pass
+        except Exception:
+            pass
+
+
+def clear_llm_cache(provider: Optional[ModelProvider] = None) -> None:
+    """
+    Clear the LLM cache, optionally for a specific provider only.
+
+    Args:
+        provider: If specified, only clear cache entries for this provider.
+                 If None, clear the entire cache.
+    """
+    # Before clearing, try to cleanup any Gemini clients
+    _cleanup_gemini_clients()
+
+    if provider is None:
+        # Clear entire cache
+        _llm_cache.clear()
+    else:
+        # For simplicity, just clear all when provider is specified
+        _llm_cache.clear()
+
+
 def get_llm(role: LLMRole, config: Optional[AgentConfig] = None) -> LLM:
     """
     Get the LLM for the specified role, using the provided config
@@ -159,12 +213,19 @@ def get_llm(role: LLMRole, config: Optional[AgentConfig] = None) -> LLM:
                 "google_genai not available. Install with: pip install llama-index-llms-google-genai"
             ) from e
         import google.genai.types as google_types
-        generation_config = google_types.GenerateContentConfig(
-            temperature=0.0,
-            seed=123,
-            max_output_tokens=max_tokens,
-            thinking_config=google_types.ThinkingConfig(thinking_budget=0, include_thoughts=False),
-        )
+
+        # Only include thinking_config for models that support it (Gemini 2.5+)
+        config_kwargs = {
+            "temperature": 0.0,
+            "seed": 123,
+            "max_output_tokens": max_tokens,
+        }
+        # Gemini models support thinking
+        if "gemini" in model_name.lower():
+            config_kwargs["thinking_config"] = google_types.ThinkingConfig(
+                thinking_budget=0, include_thoughts=False
+            )
+        generation_config = google_types.GenerateContentConfig(**config_kwargs)
         llm = GoogleGenAI(
             model=model_name,
             temperature=0,
@@ -182,11 +243,13 @@ def get_llm(role: LLMRole, config: Optional[AgentConfig] = None) -> LLM:
             ) from e
         additional_kwargs = {"seed": 42}
         if model_name in [
-            "deepseek-ai/DeepSeek-V3.1", "openai/gpt-oss-120b",
-            "deepseek-ai/DeepSeek-R1", "Qwen/Qwen3-235B-A22B-Thinking-2507"
-            "openai/gpt-oss-120b", "openai/gpt-oss-20b",
+            "deepseek-ai/DeepSeek-V3.1",
+            "deepseek-ai/DeepSeek-R1",
+            "Qwen/Qwen3-235B-A22B-Thinking-2507",
+            "openai/gpt-oss-120b",
+            "openai/gpt-oss-20b",
         ]:
-            additional_kwargs['reasoning_effort'] = "low"
+            additional_kwargs["reasoning_effort"] = "low"
         llm = TogetherLLM(
             model=model_name,
             temperature=0,
@@ -206,6 +269,8 @@ def get_llm(role: LLMRole, config: Optional[AgentConfig] = None) -> LLM:
             temperature=0,
             is_function_calling_model=True,
             max_tokens=max_tokens,
+            reasoning_format="hidden",
+            reasoning_effort="low",
         )
     elif model_provider == ModelProvider.BEDROCK:
         try:
@@ -247,6 +312,8 @@ def get_llm(role: LLMRole, config: Optional[AgentConfig] = None) -> LLM:
             raise ValueError(
                 "Private LLM requires both private_llm_api_base and private_llm_api_key to be set in AgentConfig."
             )
+        # Use private_llm_max_tokens if specified (non-zero), otherwise fall back to default
+        private_max_tokens = config.private_llm_max_tokens if config.private_llm_max_tokens > 0 else max_tokens
         llm = OpenAILike(
             model=model_name,
             temperature=0,
@@ -254,7 +321,7 @@ def get_llm(role: LLMRole, config: Optional[AgentConfig] = None) -> LLM:
             is_chat_model=True,
             api_base=config.private_llm_api_base,
             api_key=config.private_llm_api_key,
-            max_tokens=max_tokens,
+            max_tokens=private_max_tokens,
         )
 
     else:
